unit Settings_General;

(******************************************************************************)
(*                                                                            *)
(* Hermes General Settings Dialog Box                                         *)
(* Part of Hermes SMTP/POP3 Server.                                           *)
(* Copyright(C) 2000 by Alexander J. Fanti, All Rights Reserver Worldwide.    *)
(*                                                                            *)
(* Created January 19, 2000 by Alexander J. Fanti.  See License.txt           *)
(*                                                                            *)
(* Used by: Main                                                              *)
(* Uses: DataU1, UtilU1                                                       *)
(*                                                                            *)
(* Description: This Modal dialog window allows the user to change general    *)
(*              server settings                                               *)
(*                                                                            *)
(* Revisions: 1/21/2000  AJF  Commented                                       *)
(*            2/13/2000  AJF  Added Startup options                           *)
(*            3/11/2000  AJF  Added DNS Timeout and Process Queue on Startup  *)
(*                                                                            *)
(******************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl, ShellAPI, ShlObj, ComObj, StdCtrls, Buttons, Spin, ComCtrls,
  Registry;

type
  TfrmSettings_General = class(TForm)
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    SaveDialog1: TSaveDialog;
    PageControl1: TPageControl;
    tsGeneral: TTabSheet;
    tsStartup: TTabSheet;
    tsLog: TTabSheet;
    Label1: TLabel;
    txtServerName: TEdit;
    Label2: TLabel;
    txtMailboxPath: TEdit;
    btnSelectMailbox: TButton;
    btnSelectQueue: TButton;
    txtQueuePath: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    txtLogFile: TEdit;
    btnSelectLogFile: TButton;
    lblLogLevel: TLabel;
    lstLogLevel: TComboBox;
    cbxStartMinimized: TCheckBox;
    cbxAutoStart_SmtpServer: TCheckBox;
    cbxAutoStart_SmtpAgent: TCheckBox;
    cbxAutoStart_Pop3Server: TCheckBox;
    speDNSTimeout: TSpinEdit;
    Label5: TLabel;
    cbxStartProcessQueue: TCheckBox;
    cbxSpyMessageContent: TCheckBox;
    lstDNSServerAddress: TComboBox;
    tsBanner: TTabSheet;
    optBanner_NVS: TRadioButton;
    optBanner_NS: TRadioButton;
    optBanner_S: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure btnSelectMailboxClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnSelectQueueClick(Sender: TObject);
    procedure btnSelectLogFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmSettings_General: TfrmSettings_General;

implementation

uses DataU1, UtilU1;

{$R *.DFM}

procedure TfrmSettings_General.FormCreate(Sender: TObject);
var
  x : Longint;
  DNSList : TStringList;
begin
  DNSList := TStringList.Create;
  FetchDNSList(DNSList);
  lstDNSServerAddress.Items.Clear;
  for x := 0 to DNSList.Count -1 do lstDNSServerAddress.Items.Add(DNSList[x]);
  DNSList.Free;
end;

procedure TfrmSettings_General.FormShow(Sender: TObject);
begin
  PageControl1.ActivePage := tsGeneral;
  // Display General settings in dialog
  txtServerName.Text              := TheServerInfo.ServerName;
  cbxStartMinimized.Checked       := TheServerInfo.StartMinimized;
  cbxAutoStart_SmtpServer.Checked := TheServerInfo.AutoStart_SmtpServer;
  cbxAutoStart_SmtpAgent.Checked  := TheServerInfo.AutoStart_SmtpAgent;
  cbxAutoStart_Pop3Server.Checked := TheServerInfo.AutoStart_Pop3Server;
  cbxStartProcessQueue.Checked    := TheServerInfo.ProcessQueueOnStartup;
  txtMailboxPath.Text             := TheServerInfo.MailboxPath;
  txtQueuePath.Text               := TheServerInfo.MailQueuePath;
  lstDNSServerAddress.Text        := TheServerInfo.DNSServerAddress;
  speDNSTimeout.Value             := TheServerInfo.DNSServerTimeout;
  txtLogFile.Text                 := TheServerInfo.LogFile;
  lstLogLevel.ItemIndex           := TheServerInfo.LogLevel;
  cbxSpyMessageContent.Checked    := TheServerInfo.LogSpyMessageContent;

  case TheServerInfo.Banner_Level of
    bannerlevel_NameVersionService : optBanner_NVS.Checked := True;
    bannerlevel_NameService        : optBanner_NS.Checked := True;
    bannerlevel_Service            : optBanner_S.Checked := True;
    else optBanner_NVS.Checked := True;
  end;

end;
procedure TfrmSettings_General.btnOKClick(Sender: TObject);
begin
  // Read General settings from dialog
  if(trim(txtServerName.Text) = '')then
    begin
      txtServerName.SetFocus;
      Abort;
    end;  
  TheServerInfo.ServerName := Trim(txtServerName.Text);
  TheServerInfo.StartMinimized := cbxStartMinimized.Checked;
  TheServerInfo.AutoStart_SmtpServer := cbxAutoStart_SmtpServer.Checked;
  TheServerInfo.AutoStart_SmtpAgent := cbxAutoStart_SmtpAgent.Checked;
  TheServerInfo.AutoStart_Pop3Server := cbxAutoStart_Pop3Server.Checked;
  TheServerInfo.ProcessQueueOnStartup := cbxStartProcessQueue.Checked;
  TheServerInfo.MailboxPath := Trim(txtMailboxPath.Text);
  TheServerInfo.MailListPath := Trim(txtMailboxPath.Text);
  TheServerInfo.MailQueuePath := Trim(txtQueuePath.Text);
  TheServerInfo.DNSServerAddress := '';
  if IsDomainDottedIP(Trim(lstDNSServerAddress.Text)) then
    TheServerInfo.DNSServerAddress := Trim(lstDNSServerAddress.Text)
    else ShowMessage('DNS Server Address Invalid');
  TheServerInfo.DNSServerTimeout     := speDNSTimeout.Value;
  TheServerInfo.LogFile              := Trim(txtLogFile.Text);
  TheServerInfo.LogLevel             := lstLogLevel.ItemIndex;
  TheServerInfo.LogSpyMessageContent := cbxSpyMessageContent.Checked;

  TheServerInfo.Banner_Level := bannerlevel_NameVersionService;
  if optBanner_NVS.Checked then TheServerInfo.Banner_Level := bannerlevel_NameVersionService;
  if optBanner_NS.Checked  then TheServerInfo.Banner_Level := bannerlevel_NameService;
  if optBanner_S.Checked   then TheServerInfo.Banner_Level := bannerlevel_Service;

end;

procedure TfrmSettings_General.btnSelectMailboxClick(Sender: TObject);
var
  TitleName : string;
  lpItemID : PItemIDList;
  BrowseInfo : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  // Select a folder for MailBox
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  BrowseInfo.hwndOwner := Self.Handle;
  BrowseInfo.pidlRoot := nil;  // Desktop
  BrowseInfo.pszDisplayName := @DisplayName;
  TitleName := 'Please specify a directory';
  BrowseInfo.lpszTitle := PChar(TitleName);
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemId <> nil then begin
    SHGetPathFromIDList(lpItemID, TempPath);
    txtMailboxPath.Text := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;

procedure TfrmSettings_General.btnSelectQueueClick(Sender: TObject);
var
  TitleName : string;
  lpItemID : PItemIDList;
  BrowseInfo : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  // Select a folder for Queue Path
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  BrowseInfo.hwndOwner := Self.Handle;
  BrowseInfo.pidlRoot := nil;  // Desktop
  BrowseInfo.pszDisplayName := @DisplayName;
  TitleName := 'Please specify a directory';
  BrowseInfo.lpszTitle := PChar(TitleName);
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemId <> nil then begin
    SHGetPathFromIDList(lpItemID, TempPath);
    txtQueuePath.Text := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;

procedure TfrmSettings_General.btnSelectLogFileClick(Sender: TObject);
begin
  //Choose a Log File
  if SaveDialog1.Execute then txtLogFile.Text := SaveDialog1.Filename;
end;

end.
